@@ -27,7 +27,8 @@ mi.eval <- function(EXPR, robust, cluster, coef., vcov., df.=NULL, parallel=NULL
                 mthd.plus <- as.character(enquote(mf$cluster)[-1])
             if (nchar(mthd.plus) >= 40)
                 mthd.plus <- c("robust standard errors clustered on:",mthd.plus)
-            mthd.plus <- paste("robust standard errors clustered on",mthd.plus)
+            else
+                mthd.plus <- paste("robust standard errors clustered on",mthd.plus)
         }
         else if (robust) {
             vcov.expr[[1L]] <- quote(sandwich::vcovHC)
@@ -90,10 +91,15 @@ mi.eval <- function(EXPR, robust, cluster, coef., vcov., df.=NULL, parallel=NULL
         if (!parallel)
             warning("Can't load \"parallel\" parallel. Using serial computation.")
     }
-    if (parallel)
+    if (parallel) {
         imp.list <- parallel::mclapply(1:num.imp, imp.info$sub, mf=mf, m=m, ...)
-    else
+        if (any(sapply(imp.list, inherits, what="try-error"))) {
+            stop(imp.list[which(sapply(imp.list, inherits, what="try-error"))[1]],
+                 call.=FALSE)
+        }
+    } else {
         imp.list <- lapply(1:num.imp, imp.info$sub, mf=mf, m=m, ...)
+    }
     names(imp.list) <- imp.info$names
     if (is.null(df.)) {
         if (all(sapply(imp.list, inherits, what="glm") | sapply(imp.list, inherits, what="glmerMod")))
@@ -101,8 +107,11 @@ mi.eval <- function(EXPR, robust, cluster, coef., vcov., df.=NULL, parallel=NULL
         else
             df. <- df.residual
     }
-    if (is.function(df.))
-        df. <- min(sapply(imp.list, df.))
+    if (is.function(df.)) {
+        df. <- try(min(sapply(imp.list, df.)), silent=TRUE)
+        if (inherits(df., "try-error") || !is.numeric(df.))
+            df. <- Inf
+    }
     if (is.null(df.))
         df. <- Inf
     coeflist <- suppressWarnings(lapply(1:length(imp.list), function(i, expr) { expr[[2]] <- substitute(imp.list[[i]], list(i=i)); eval(expr) }, expr=coef.expr))
@@ -151,6 +160,19 @@ mi.eval <- function(EXPR, robust, cluster, coef., vcov., df.=NULL, parallel=NULL
             m.out$nobs <- imp.nobs
         }
     }
+    imp.family <- try(lapply(imp.list, family), silent=TRUE)
+    if (!inherits(imp.family, "try-error") && length(unique(imp.family)) == 1)
+        m.out$family <- imp.family[[1L]]
+    imp.class <- try(lapply(imp.list, class), silent=TRUE)
+    if (!inherits(imp.class, "try-error") && length(unique(imp.class)) == 1)
+        m.out$model.class <- imp.class[[1L]]
+    imp.terms <- try(lapply(imp.list, terms), silent=TRUE)
+    if (!inherits(imp.terms, "try-error")) {
+        for (i in 1:length(imp.terms))
+            attr(imp.terms[[i]], ".Environment") <- NULL
+        if (length(unique(imp.terms)) == 1)
+            m.out$terms <- imp.terms[[1L]]
+    }
     imp.resid <- try(sapply(imp.list, function(x) x$residuals), silent=TRUE)
     if (!inherits(imp.resid, "try-error")) {
         if (is.matrix(imp.resid))
@@ -221,3 +243,37 @@ extract.mi.estimates <- function(model, include.nobs = TRUE, include.imp = TRUE)
         gof.decimal = sapply(gof, is.numeric) & !sapply(gof, is.integer)
     ))
 }
+
+predict.mi.estimates <- function (object, newdata = NULL, type = c("link", "response", "terms"), se.fit = FALSE, terms = NULL, na.action = na.pass, ...) {
+    type <- match.arg(type)    
+    if (missing(newdata))
+        stop("The predict method for mi.estimates objects is only implemented when newdata is specified")
+    if (type == "terms")
+        stop("The predict method for mi.estimates objects is not yet implemented for type=\"terms\"")
+    if (!any(c("glm","lm") %in% object$model.class))
+        stop("The predict method for mi.estimates objects is only implemented for those based on glm and lm objects")
+    Terms <- delete.response(object$terms)
+    m <- model.frame(Terms, newdata, na.action = na.action)
+    X <- model.matrix(Terms, m)
+    offset <- rep(0, nrow(X))
+    if (!is.null(off.num <- attr(object$terms, "offset"))) 
+        for (i in off.num)
+            offset <- offset + eval(attr(object$terms, "variables")[[i + 1]], newdata)
+    if (!is.null(object$call$offset))
+        offset <- offset + eval(object$call$offset, newdata)
+    predictor <- as.vector(X %*% coef(object))
+    if (!is.null(offset)) 
+        predictor <- predictor + offset
+    if (type == "response" && !is.null(object$family))
+        predictor <- object$family$linkinv(predictor)
+    names(predictor) <- rownames(X)
+    if (se.fit) {
+        predictor.se <- as.vector(sqrt(apply(X * (X %*% vcov(object)), 1, sum)))
+        if (type == "response" && !is.null(object$family))
+            predictor.se <- predictor.se * abs(object$family$mu.eta(predictor))
+        names(predictor.se) <- rownames(X)
+        return(list(fit = predictor, se.fit = predictor.se))
+    } else {
+        return(predictor)
+    }
+}
